@@ -12,12 +12,14 @@ from sarus_data_spec.variant_constraint import VariantConstraint
 # pylint: disable=unused-variable
 
 PACKAGE_NAME: Final[str] = 'sarus_data_spec'
-VERSION: Final[str] = '2.1.0'
+VERSION: Final[str] = '2.7.2'
 
 try:
-    # The local context is absent from the public release
     import sarus_data_spec.context.worker as sw
 
     push_global_context(sw.WorkerContext())
-except ModuleNotFoundError:
-    pass
+except ModuleNotFoundError as exception:
+    if exception.name == 'sarus_data_spec.context.worker':
+        pass  # The worker context is absent from the public release
+    else:
+        raise exception
