@@ -3,7 +3,6 @@ from __future__ import annotations
 from os.path import basename
 from typing import (
     TYPE_CHECKING,
-    Any,
     AsyncIterator,
     Collection,
     Dict,
@@ -34,17 +33,10 @@ try:
 except ModuleNotFoundError:
     warnings.warn('SqlAlchemy not found, sql operations not available')
 
-try:
-    from sarus_differential_privacy.protobuf.private_query_pb2 import (
-        PrivateQuery,
-    )
-except ModuleNotFoundError:
-    pass  # warning raised in typing.py
-
 
 from sarus_data_spec.base import Referring
-from sarus_data_spec.constants import DATASET_SLUGNAME, PRIVATE_QUERY
-from sarus_data_spec.protobuf.utilities import dejson, to_base64
+from sarus_data_spec.constants import DATASET_SLUGNAME
+from sarus_data_spec.protobuf.utilities import to_base64
 from sarus_data_spec.scalar import Scalar
 from sarus_data_spec.transform import Transform
 import sarus_data_spec.protobuf as sp
@@ -123,42 +115,21 @@ class Dataset(Referring[sp.Dataset]):
         return not self.is_transformed()
 
     def sql(
-        self, query: str, dialect: Optional[st.SQLDialect] = None
-    ) -> List[Dict[str, Any]]:
-
+        self,
+        query: t.Union[str, t.Mapping[t.Union[str, t.Tuple[str, ...]], str]],
+        dialect: Optional[st.SQLDialect] = None,
+        batch_size: int = 10000,
+    ) -> Iterator[pa.RecordBatch]:
         """Executes the sql method on the dataset"""
-        return self.manager().sql(self, query, dialect)
+        return self.manager().sql(self, query, dialect, batch_size)
 
-    def sources(self) -> Set[st.Dataset]:
+    def sources(
+        self, type_name: t.Optional[str] = sp.type_name(sp.Dataset)
+    ) -> Set[st.DataSpec]:
         """Returns the set of non-transformed datasets that are parents
         of the current dataset"""
-
-        class Sources(st.Visitor):
-            visited: Set[st.DataSpec] = set()
-            results: Set[st.Dataset] = set()
-
-            def transformed(
-                self,
-                visited: st.DataSpec,
-                transform: st.Transform,
-                *arguments: st.DataSpec,
-                **named_arguments: st.DataSpec,
-            ) -> None:
-                if visited not in self.visited:
-                    for argument in arguments:
-                        argument.accept(self)
-                    for _, argument in named_arguments.items():
-                        argument.accept(self)
-                    self.visited.add(visited)
-
-            def other(self, visited: st.DataSpec) -> None:
-                if visited.prototype() == sp.Dataset:
-                    dataset = cast(st.Dataset, visited)
-                    self.results.add(dataset)
-
-        visitor = Sources()
-        self.accept(visitor)
-        return visitor.results
+        sources = self.storage().sources(self, type_name=type_name)
+        return sources
 
     def status(self, task_names: t.List[str]) -> t.Optional[st.Status]:
         """This method return a status that contains all the
@@ -169,6 +140,10 @@ class Dataset(Referring[sp.Dataset]):
 
         # collects last status for each task. This if needed will also update
         # them if any synchronization is needed.
+        if type(task_names) not in [list, set, tuple]:
+            raise TypeError(
+                f"Invalid task_names passed to dataset.status {task_names}"
+            )
         statuses = [
             self.manager().status(self, task_name) for task_name in task_names
         ]
@@ -202,9 +177,7 @@ class Dataset(Referring[sp.Dataset]):
             ),
         )
 
-    def to_arrow(
-        self, batch_size: int = 10000
-    ) -> st.ContextManagerIterator[pa.RecordBatch]:
+    def to_arrow(self, batch_size: int = 10000) -> t.Iterator[pa.RecordBatch]:
         return self.manager().to_arrow(self, batch_size)
 
     async def async_to_arrow(
@@ -212,6 +185,9 @@ class Dataset(Referring[sp.Dataset]):
     ) -> AsyncIterator[pa.RecordBatch]:
         return await self.manager().async_to_arrow(self, batch_size)
 
+    def to_sql(self) -> None:
+        return self.manager().to_sql(self)
+
     def parents(
         self,
     ) -> Tuple[List[st.DataSpec], Dict[str, st.DataSpec]]:
@@ -235,7 +211,7 @@ class Dataset(Referring[sp.Dataset]):
     def variant(
         self,
         kind: st.ConstraintKind,
-        public_context: List[str] = [],
+        public_context: Collection[str] = (),
         privacy_limit: Optional[st.PrivacyLimit] = None,
     ) -> Optional[st.DataSpec]:
         return (
@@ -247,7 +223,7 @@ class Dataset(Referring[sp.Dataset]):
     def variants(self) -> Collection[st.DataSpec]:
         return self.manager().query_manager().variants(self)
 
-    def private_queries(self) -> List[PrivateQuery]:
+    def private_queries(self) -> List[st.PrivateQuery]:
         """Return the list of PrivateQueries used in a Dataspec's transform.
 
         It represents the privacy loss associated with the current computation.
@@ -256,16 +232,9 @@ class Dataset(Referring[sp.Dataset]):
         to update its accountant. Note that Private Query objects are generated
         with a random uuid so that even if they are submitted multiple times to
         an account, they are only accounted once (ask @cgastaud for more on
-        accounting)."""
-        attribute = self.attribute(name=PRIVATE_QUERY)
-        if attribute is None:
-            return []
-
-        private_query_str = attribute[PRIVATE_QUERY]
-        return [
-            cast(PrivateQuery, dejson(q))
-            for q in json.loads(private_query_str)
-        ]
+        accounting).
+        """
+        return self.manager().query_manager().private_queries(self)
 
     def spec(self) -> str:
         return cast(str, self.protobuf().spec.WhichOneof('spec'))
@@ -378,6 +347,9 @@ class Dataset(Referring[sp.Dataset]):
     def attribute(self, name: str) -> t.Optional[st.Attribute]:
         return self.manager().attribute(name=name, dataspec=self)
 
+    def attributes(self, name: str) -> t.List[st.Attribute]:
+        return self.manager().attributes(name=name, dataspec=self)
+
 
 # Builders
 def transformed(
@@ -387,7 +359,6 @@ def transformed(
     dataspec_name: Optional[str] = None,
     **named_arguments: st.DataSpec,
 ) -> st.DataSpec:
-
     if dataspec_type is None:
         dataspec_type, attach_info_callback = transform.infer_output_type(
             *arguments, **named_arguments
