@@ -14,6 +14,7 @@ from sarus_data_spec.manager.asyncio.utils import sync, sync_iterator
 from sarus_data_spec.manager.base import Base
 from sarus_data_spec.manager.typing import Computation, Manager
 from sarus_data_spec.schema import Schema
+import sarus_data_spec.protobuf as sp
 import sarus_data_spec.status as stt
 import sarus_data_spec.typing as st
 
@@ -53,7 +54,15 @@ class BaseAsyncManager(Base):
     """
 
     def dataspec_computation(self, dataspec: st.DataSpec) -> Computation:
-        """Return the Computation for getting the dataspec's value."""
+        """Return the Computation for getting the dataspec's value. If sql is
+        is true, the sqlcomputation will be returned.
+        """
+        raise NotImplementedError
+
+    def sql_computation(self) -> Computation:
+        """Returns the SQL Computation of the manager. The aim of the method is
+        is to reduce code duplication between API and Worker manager
+        """
         raise NotImplementedError
 
     def prepare(self, dataspec: st.DataSpec) -> None:
@@ -105,7 +114,7 @@ class BaseAsyncManager(Base):
 
     def to_arrow(
         self, dataset: st.Dataset, batch_size: int
-    ) -> st.ContextManagerIterator[pa.RecordBatch]:
+    ) -> t.Iterator[pa.RecordBatch]:
         return sync_iterator(
             self.async_to_arrow(dataset=dataset, batch_size=batch_size)
         )
@@ -129,6 +138,72 @@ class BaseAsyncManager(Base):
     def to_parquet(self, dataset: st.Dataset) -> None:
         sync(self.async_to_parquet(dataset=dataset))
 
+    def to_sql(self, dataset: st.Dataset) -> None:
+        sync(self.async_to_sql(dataset=dataset))
+
+    def sql_prepare(self, dataset: st.Dataset) -> None:
+        """SQL prepare dataset."""
+        sync(self.async_sql_prepare(dataset))
+
+    async def async_sql_prepare(self, dataset: st.Dataset) -> None:
+        """SQL prepare the dataset synchronously"""
+        computation = t.cast(
+            BaseComputation[t.AsyncIterator[pa.RecordBatch]],
+            self.sql_computation(),
+        )
+        await computation.complete_task(dataset)
+
+    async def async_sql_prepare_parents(self, dataset: st.Dataset) -> None:
+        """SQL prepare all the parents of a dataset: calling async_sql_prepare
+        for dataset parents and async_prepare for scalar parents.
+        """
+        args, kwargs = dataset.parents()
+        parents = list(args) + list(kwargs.values())
+        coros = [
+            self.async_sql_prepare(t.cast(st.Dataset, parent))
+            if dataset.prototype() == sp.Dataset
+            else self.async_prepare(parent)
+            for parent in parents
+        ]
+        # here, if many parents potentially fail, we want to be sure
+        # that all of them do it, and not only the first one (to
+        # modify all statuses accordingly).
+        # After, we only raise the first exception for the child.
+
+        results = await asyncio.gather(*coros, return_exceptions=True)
+        exceptions = [
+            element for element in results if isinstance(element, Exception)
+        ]
+        if len(exceptions) == 0:
+            return
+        raise error_aggregation(exceptions)
+
+    def sql(
+        self,
+        dataset: st.Dataset,
+        query: t.Union[str, t.Mapping[t.Union[str, t.Tuple[str, ...]], str]],
+        dialect: t.Optional[st.SQLDialect] = None,
+        batch_size: int = 10000,
+    ) -> t.Iterator[pa.RecordBatch]:
+        return sync_iterator(
+            self.async_sql(dataset, query, dialect, batch_size)
+        )
+
+    async def async_sql(
+        self,
+        dataset: st.Dataset,
+        query: t.Union[str, t.Mapping[t.Union[str, t.Tuple[str, ...]], str]],
+        dialect: t.Optional[st.SQLDialect] = None,
+        batch_size: int = 10000,
+    ) -> t.AsyncIterator[pa.RecordBatch]:
+        computation = t.cast(
+            BaseComputation[t.AsyncIterator[pa.RecordBatch]],
+            self.sql_computation(),
+        )
+        return await computation.task_result(
+            dataset, query=query, dialect=dialect, batch_size=batch_size
+        )
+
     def cache_scalar(self, scalar: st.Scalar) -> None:
         sync(self.async_cache_scalar(scalar=scalar))
 
@@ -149,13 +224,15 @@ class BaseAsyncManager(Base):
             t.Dict[st.Path, st.Path], sync(self.async_foreign_keys(dataset))
         )
 
+    def primary_keys(self, dataset: st.Dataset) -> t.List[st.Path]:
+        return t.cast(t.List[st.Path], sync(self.async_primary_keys(dataset)))
+
     async def async_links(self, dataset: st.Dataset) -> t.Any:
         raise NotImplementedError
 
     async def async_to_tensorflow(
         self, dataset: st.Dataset
     ) -> tf.data.Dataset:
-
         root_dir = os.path.join(
             self.parquet_dir(), "tfrecords", dataset.uuid()
         )
@@ -262,7 +339,7 @@ class BaseComputation(Computation[T]):
                 },
             )
             raise stt.DataSpecErrorStatus(
-                relaunch=exception.relaunch, error_msg=traceback.format_exc()
+                (exception.relaunch, traceback.format_exc())
             )
         except Exception:
             stt.error(
@@ -274,9 +351,7 @@ class BaseComputation(Computation[T]):
                     'relaunch': str(False),
                 },
             )
-            raise stt.DataSpecErrorStatus(
-                relaunch=False, error_msg=traceback.format_exc()
-            )
+            raise stt.DataSpecErrorStatus((False, traceback.format_exc()))
 
     async def read_ready_result(
         self,
@@ -345,10 +420,40 @@ class BaseComputation(Computation[T]):
             stt.clear_task(dataspec=dataspec, task=self.task_name)
             return await self.complete_task(dataspec=dataspec)
         raise stt.DataSpecErrorStatus(
-            relaunch=stage.properties()["relaunch"] == str(True),
-            error_msg=stage.properties()["message"],
+            (
+                stage.properties()["relaunch"] == str(True),
+                stage.properties()["message"],
+            )
         )
 
+    async def wait_for_computation(
+        self,
+        dataspec: st.DataSpec,
+        current_stage: str,
+        timeout: int = 300,
+        max_delay: int = 10,
+    ) -> st.Stage:
+        """Utility to wait for the availability of a computation, by polling
+        its status for at most `timeout` seconds, with a period of at most
+        `max_delay` seconds between each attempt. Delay grows exponentially
+        at first."""
+        delay = min(1, timeout, max_delay)
+        total_wait = 0
+        while total_wait < timeout:
+            status = self.status(dataspec=dataspec)
+            assert status
+            stage = status.task(self.task_name)
+            assert stage
+            if stage.stage() == current_stage:
+                logger.debug(f'POLLING {self.task_name} {dataspec.uuid()}')
+                await asyncio.sleep(delay)
+                total_wait += delay
+                delay = min(2 * delay, max_delay, timeout - total_wait)
+            else:
+                break
+        assert stage
+        return stage
+
 
 class ErrorCatchingAsyncIterator:
     """Wrap an AsyncIterator and catches potential errors.
@@ -387,7 +492,7 @@ class ErrorCatchingAsyncIterator:
                 },
             )
             raise stt.DataSpecErrorStatus(
-                relaunch=exception.relaunch, error_msg=traceback.format_exc()
+                (exception.relaunch, traceback.format_exc())
             )
         except Exception:
             stt.error(
@@ -399,9 +504,7 @@ class ErrorCatchingAsyncIterator:
                     'relaunch': str(False),
                 },
             )
-            raise stt.DataSpecErrorStatus(
-                relaunch=False, error_msg=traceback.format_exc()
-            )
+            raise stt.DataSpecErrorStatus((False, traceback.format_exc()))
 
         else:
             return batch
