@@ -26,6 +26,12 @@ class TypedWorkerManager(Manager, t.Protocol):
     ) -> t.AsyncIterator[pa.RecordBatch]:
         ...
 
+    def computation_timeout(self, dataspec: st.DataSpec) -> int:
+        ...
+
+    def computation_max_delay(self, dataspec: st.DataSpec) -> int:
+        ...
+
 
 class WorkerComputation(BaseComputation[T]):
     def __init__(self, manager: TypedWorkerManager):
@@ -48,19 +54,12 @@ class WorkerComputation(BaseComputation[T]):
         of the task. After a given timeout, an error is raised.
         """
 
-        for _ in range(100):
-            status = stt.last_status(
-                dataspec=dataspec, task=self.task_name, manager=self.manager()
-            )
-            assert status
-            logger.debug(f'POLLING {self.task_name} {dataspec.uuid()}')
-            stage = status.task(self.task_name)
-            assert stage
-            if not stage.processing():
-                break
-            await asyncio.sleep(1)
-
-        assert stage
+        stage = await self.wait_for_computation(
+            dataspec=dataspec,
+            current_stage='processing',
+            timeout=self.manager().computation_timeout(dataspec),
+            max_delay=self.manager().computation_max_delay(dataspec),
+        )
         if stage.processing():
             stt.error(
                 dataspec=dataspec,
@@ -73,16 +72,17 @@ class WorkerComputation(BaseComputation[T]):
                 },
             )
             raise stt.DataSpecErrorStatus(
-                relaunch=True,
-                error_msg="TimeOutError:Processing time out for task"
-                f" {self.task_name} on dataspec {dataspec}",
+                (
+                    True,
+                    "TimeOutError:Processing time out for task"
+                    f" {self.task_name} on dataspec {dataspec}",
+                )
             )
         # if the stage is an error, it is complete_task
         # that decides what to do via the error_policy
         return await self.complete_task(dataspec)
 
     def launch_task(self, dataspec: st.DataSpec) -> t.Optional[t.Awaitable]:
-
         status = stt.last_status(
             dataspec=dataspec, manager=self.manager(), task=self.task_name
         )
