@@ -166,7 +166,7 @@ class Base(manager_typing.Manager):
 
     def to_arrow(
         self, dataset: st.Dataset, batch_size: int
-    ) -> st.ContextManagerIterator[pa.RecordBatch]:
+    ) -> t.Iterator[pa.RecordBatch]:
         raise NotImplementedError
 
     def to_pandas(self, dataset: st.Dataset) -> pd.DataFrame:
@@ -227,9 +227,25 @@ class Base(manager_typing.Manager):
     def sql(
         self,
         dataset: st.Dataset,
-        query: str,
+        query: t.Union[str, t.Mapping[t.Union[str, t.Tuple[str, ...]], str]],
         dialect: t.Optional[st.SQLDialect] = None,
-    ) -> t.List[t.Dict[str, t.Any]]:
+        batch_size: int = 10000,
+    ) -> t.Iterator[pa.RecordBatch]:
+        raise NotImplementedError
+
+    async def async_sql(
+        self,
+        dataset: st.Dataset,
+        query: t.Union[str, t.Mapping[t.Union[str, t.Tuple[str, ...]], str]],
+        dialect: t.Optional[st.SQLDialect] = None,
+        batch_size: int = 10000,
+    ) -> t.AsyncIterator[pa.RecordBatch]:
+        raise NotImplementedError
+
+    def to_sql(self, dataset: st.Dataset) -> None:
+        raise NotImplementedError
+
+    async def async_to_sql(self, dataset: st.Dataset) -> None:
         raise NotImplementedError
 
     def foreign_keys(self, dataset: st.Dataset) -> t.Dict[st.Path, st.Path]:
@@ -239,8 +255,7 @@ class Base(manager_typing.Manager):
         raise NotImplementedError
 
     def is_big_data(self, dataset: st.Dataset) -> bool:
-
-        status = stt.last_status(dataset, task=BIG_DATA_TASK)
+        status = self.status(dataset, task_name=BIG_DATA_TASK)
 
         if status is not None:
             # check if big_data_present
@@ -256,6 +271,8 @@ class Base(manager_typing.Manager):
                 'Found source dataset without any big data status'
             )
         else:
+            if dataset.transform().is_external():
+                return False
             parents_list, parents_dict = dataset.parents()
             ds_args = [
                 element
@@ -323,6 +340,9 @@ class Base(manager_typing.Manager):
     def is_cached(self, dataspec: st.DataSpec) -> bool:
         raise NotImplementedError
 
+    def is_pushed_to_sql(self, dataspec: st.DataSpec) -> bool:
+        raise NotImplementedError
+
     async def async_primary_keys(self, dataset: st.Dataset) -> t.List[st.Path]:
         raise NotImplementedError
 
@@ -334,7 +354,6 @@ class Base(manager_typing.Manager):
     def attribute(
         self, name: str, dataspec: st.DataSpec
     ) -> t.Optional[st.Attribute]:
-
         attributes = t.cast(
             t.Set[st.Attribute],
             self.storage().referring(
@@ -353,9 +372,23 @@ class Base(manager_typing.Manager):
 
         raise ValueError('There are two attributes with the same name')
 
+    def attributes(
+        self, name: str, dataspec: st.DataSpec
+    ) -> t.List[st.Attribute]:
+        attributes = t.cast(
+            t.Set[st.Attribute],
+            self.storage().referring(
+                dataspec, type_name=sp.type_name(sp.Attribute)
+            ),
+        )
+        return [element for element in attributes if element.name() == name]
+
     def links(self, dataset: st.Dataset) -> st.Links:
         raise NotImplementedError
 
+    def sql_pushing_schema_prefix(self, dataset: st.Dataset) -> str:
+        raise NotImplementedError
+
 
 def check_transform_big_data(
     transform: st.Transform,
@@ -368,6 +401,7 @@ def check_transform_big_data(
     affect the size, it should return True
     """
     status = stt.last_status(parent_dataset, task=BIG_DATA_TASK)
+    assert status
     big_data_threshold = int(
         status.task(BIG_DATA_TASK).properties()[  # type:ignore
             BIG_DATA_THRESHOLD
@@ -377,17 +411,34 @@ def check_transform_big_data(
         THRESHOLD_TYPE
     ]
 
-    parent_n_lines = int(
-        status.task(BIG_DATA_TASK).properties()[DATASET_N_LINES]  # type:ignore
-    )
-    parent_bytes = int(
-        status.task(BIG_DATA_TASK).properties()[DATASET_N_BYTES]  # type:ignore
-    )
+    parent_n_lines_str = status.task(
+        BIG_DATA_TASK
+    ).properties()[  # type:ignore
+        DATASET_N_LINES
+    ]
+    if parent_n_lines_str == '':
+        parent_n_lines = 0
+    else:
+        parent_n_lines = int(parent_n_lines_str)
+
+    parent_bytes_str = status.task(BIG_DATA_TASK).properties()[  # type:ignore
+        DATASET_N_BYTES
+    ]
+    if parent_bytes_str == '':
+        parent_bytes = 0
+    else:
+        parent_bytes = int(parent_bytes_str)
 
     transform_name = transform.name()
-    if transform_name == 'Sample':
-        if transform.protobuf().spec.sample.HasField('fraction'):
-            fraction = transform.protobuf().spec.sample.fraction
+    if transform_name in ('Sample', 'DifferentiatedSample'):
+        transform_type = transform.protobuf().spec.WhichOneof('spec')
+        assert transform_type
+        if getattr(transform.protobuf().spec, transform_type).HasField(
+            'fraction'
+        ):
+            fraction = getattr(
+                transform.protobuf().spec, transform_type
+            ).fraction
             new_bytes = int(fraction * parent_bytes)
             n_lines = int(fraction * parent_n_lines)
 
@@ -399,7 +450,7 @@ def check_transform_big_data(
                     threshold_kind,
                 )
         else:
-            n_lines = transform.protobuf().spec.sample.size
+            n_lines = getattr(transform.protobuf().spec, transform_type).size
             new_bytes = int(n_lines * parent_bytes / parent_n_lines)
 
             if threshold_kind == DATASET_N_BYTES:
@@ -437,6 +488,9 @@ def check_transform_big_data(
             int(synthetic_size * parent_bytes / parent_n_lines),
             threshold_kind,
         )
+    elif transform_name == "select_sql":
+        # TODO https://gitlab.com/sarus-tech/sarus-data-spec/-/issues/207
+        return (False, -1, -1, threshold_kind)
 
     else:
         # other transforms do not affect size
