@@ -7,10 +7,7 @@ try:
 except ModuleNotFoundError:
     pass  # error message in typing.py
 
-from .utils import sarus_external_op
 
-
-@sarus_external_op
 async def imb_pipeline(
     *args: Any,
     **kwargs: Any,
@@ -18,7 +15,6 @@ async def imb_pipeline(
     return pipeline.Pipeline(*args, **kwargs)
 
 
-@sarus_external_op
 async def imb_random_under_sampler(
     *args: Any,
     **kwargs: Any,
@@ -26,7 +22,6 @@ async def imb_random_under_sampler(
     return under_sampling.RandomUnderSampler(*args, **kwargs)
 
 
-@sarus_external_op
 async def imb_smotenc(
     *args: Any,
     **kwargs: Any,
