@@ -7,7 +7,8 @@ from sarus_data_spec.bounds import bounds as bounds_builder
 from sarus_data_spec.constants import DATASET_SLUGNAME
 from sarus_data_spec.dataset import Dataset
 from sarus_data_spec.manager.ops.asyncio.processor.standard.standard_op import (  # noqa: E501
-    StandardDatasetOp,
+    StandardDatasetImplementation,
+    StandardDatasetStaticChecker,
 )
 from sarus_data_spec.manager.ops.asyncio.processor.standard.visitor_selector import (  # noqa : E501
     select_rows,
@@ -20,12 +21,7 @@ import sarus_data_spec.type as sdt
 import sarus_data_spec.typing as st
 
 
-class GetItem(StandardDatasetOp):
-    """Computes schema and arrow
-    batches for a dataspec transformed by
-    a get_item transform
-    """
-
+class GetItemStaticChecker(StandardDatasetStaticChecker):
     async def schema(self) -> st.Schema:
         parent_schema = await self.parent_schema()
         path = Path(self.dataset.transform().protobuf().spec.get_item.path)
@@ -44,6 +40,13 @@ class GetItem(StandardDatasetOp):
             name=self.dataset.properties().get(DATASET_SLUGNAME, None),
         )
 
+
+class GetItem(StandardDatasetImplementation):
+    """Computes schema and arrow
+    batches for a dataspec transformed by
+    a get_item transform
+    """
+
     async def to_arrow(
         self, batch_size: int
     ) -> t.AsyncIterator[pa.RecordBatch]:
@@ -51,48 +54,43 @@ class GetItem(StandardDatasetOp):
         path = Path(self.dataset.transform().protobuf().spec.get_item.path)
         parent_schema = await self.parent_schema()
 
-        async def async_generator(
-            parent_iter: t.AsyncIterator[pa.RecordBatch],
-        ) -> t.AsyncIterator[pa.RecordBatch]:
-            async for batch in parent_iter:
-                # TODO: what follows is really ugly and is necessary
-                # because we do not have a proper type for the
-                # protected entity, it will be removed when we
-                # switch to that formalism
-                array = convert_record_batch(
-                    record_batch=batch, _type=parent_schema.type()
+        async def get_item_func(batch: pa.RecordBatch) -> pa.Array:
+            array = convert_record_batch(
+                record_batch=batch, _type=parent_schema.type()
+            )
+            # VERY UGLY SHOULD BE REMOVED WHEN WE HAVE PROTECTED TYPE
+            if 'data' in parent_schema.type().children():
+                old_arrays = array.flatten()
+                array = array.field('data')
+                updated_array = get_items(
+                    _type=parent_schema.data_type(),
+                    array=array,
+                    path=path,
+                )
+                old_arrays[0] = updated_array
+                return pa.StructArray.from_arrays(
+                    old_arrays,
+                    names=list(parent_schema.type().children().keys()),
                 )
-                # VERY UGLY SHOULD BE REMOVED WHEN WE HAVE PROTECTED TYPE
-                if 'data' in parent_schema.type().children():
-                    old_arrays = array.flatten()
-                    array = array.field('data')
-                    updated_array = get_items(
-                        _type=parent_schema.data_type(),
-                        array=array,
-                        path=path,
-                    )
-                    old_arrays[0] = updated_array
-                    new_struct = pa.StructArray.from_arrays(
-                        old_arrays,
-                        names=list(parent_schema.type().children().keys()),
-                    )
-                    yield pa.RecordBatch.from_struct_array(new_struct)
-                else:
-                    updated_array = get_items(
-                        _type=parent_schema.data_type(),
-                        array=array,
-                        path=path,
-                    )
-                    if isinstance(updated_array, pa.StructArray):
-                        yield pa.RecordBatch.from_struct_array(updated_array)
-                    else:
-                        yield pa.RecordBatch.from_arrays(
-                            [updated_array],
-                            names=[path.to_strings_list()[0][-1]],
-                        )
 
-        return async_generator(
-            parent_iter=await previous_ds.async_to_arrow(batch_size=batch_size)
+            updated_array = get_items(
+                _type=parent_schema.data_type(),
+                array=array,
+                path=path,
+            )
+            if isinstance(updated_array, pa.StructArray):
+                return updated_array
+            return pa.StructArray.from_arrays(
+                [updated_array],
+                names=[path.to_strings_list()[0][-1]],
+            )
+
+        return self.ensure_batch_correct(
+            async_iterator=await previous_ds.async_to_arrow(
+                batch_size=batch_size
+            ),
+            batch_size=batch_size,
+            func_to_apply=get_item_func,
         )
 
     async def size(self) -> st.Size:
@@ -134,7 +132,6 @@ def get_items(array: pa.Array, path: st.Path, _type: st.Type) -> pa.Array:
             name: t.Optional[str] = None,
             properties: t.Optional[t.Mapping[str, str]] = None,
         ) -> None:
-
             if len(path.sub_paths()) > 0:
                 sub_path = path.sub_paths()[0]
                 self.batch_array = get_items(
@@ -172,7 +169,6 @@ def get_items(array: pa.Array, path: st.Path, _type: st.Type) -> pa.Array:
             name: t.Optional[str] = None,
             properties: t.Optional[t.Mapping[str, str]] = None,
         ) -> None:
-
             if len(path.sub_paths()) == 0:
                 self.batch_array = array
             else:
