@@ -4,9 +4,14 @@ import pyarrow as pa
 
 from sarus_data_spec.arrow.array import convert_record_batch
 from sarus_data_spec.bounds import bounds as bounds_builder
-from sarus_data_spec.constants import DATASET_SLUGNAME, OPTIONAL_VALUE
+from sarus_data_spec.constants import (
+    DATASET_SLUGNAME,
+    OPTIONAL_VALUE,
+    PRIMARY_KEYS,
+)
 from sarus_data_spec.manager.ops.asyncio.processor.standard.standard_op import (  # noqa: E501
-    StandardDatasetOp,
+    StandardDatasetImplementation,
+    StandardDatasetStaticChecker,
 )
 from sarus_data_spec.manager.ops.asyncio.processor.standard.visitor_selector import (  # noqa: E501
     filter_primary_keys,
@@ -21,15 +26,8 @@ import sarus_data_spec.type as sdt
 import sarus_data_spec.typing as st
 
 
-class Project(StandardDatasetOp):
-    """Computes schema and arrow
-    batches for a dataspec transformed by
-    a user_settings transform
-    """
-
-    def pep_token(
-        self, public_context: t.List[str], privacy_limit: st.PrivacyLimit
-    ) -> t.Optional[str]:
+class ProjectStaticChecker(StandardDatasetStaticChecker):
+    def pep_token(self, public_context: t.Collection[str]) -> t.Optional[str]:
         """This transform doesn't change the PEP alignment."""
         return self.parent().pep_token()
 
@@ -43,12 +41,12 @@ class Project(StandardDatasetOp):
         )
         old_properties = parent_schema.properties()
 
-        if 'primary_keys' in old_properties.keys():
+        if PRIMARY_KEYS in old_properties.keys():
             new_pks = filter_primary_keys(
-                old_properties['primary_keys'],
+                old_properties[PRIMARY_KEYS],
                 new_type,
             )
-            old_properties['primary_keys'] = new_pks  # type:ignore
+            old_properties[PRIMARY_KEYS] = new_pks  # type:ignore
 
         return schema(
             self.dataset,
@@ -58,10 +56,16 @@ class Project(StandardDatasetOp):
             name=self.dataset.properties().get(DATASET_SLUGNAME, None),
         )
 
+
+class Project(StandardDatasetImplementation):
+    """Computes schema and arrow
+    batches for a dataspec transformed by
+    a user_settings transform
+    """
+
     async def to_arrow(
         self, batch_size: int
     ) -> t.AsyncIterator[pa.RecordBatch]:
-
         schema = await self.dataset.manager().async_schema(
             dataset=self.dataset
         )
