@@ -7,7 +7,8 @@ from sarus_data_spec.bounds import bounds as bounds_builder
 from sarus_data_spec.constants import DATASET_SLUGNAME
 from sarus_data_spec.manager.asyncio.utils import async_iter
 from sarus_data_spec.manager.ops.asyncio.processor.standard.standard_op import (  # noqa: E501
-    StandardDatasetOp,
+    StandardDatasetImplementation,
+    StandardDatasetStaticChecker,
 )
 from sarus_data_spec.marginals import marginals as marg_builder
 from sarus_data_spec.schema import schema
@@ -15,12 +16,7 @@ from sarus_data_spec.size import size as size_builder
 import sarus_data_spec.typing as st
 
 
-class Shuffle(StandardDatasetOp):
-    """Computes schema and arrow
-    batches for a dataspec transformed by
-    a user_settings transform
-    """
-
+class ShuffleStaticChecker(StandardDatasetStaticChecker):
     async def schema(self) -> st.Schema:
         parent_schema = await self.parent_schema()
         return schema(
@@ -31,10 +27,16 @@ class Shuffle(StandardDatasetOp):
             name=self.dataset.properties().get(DATASET_SLUGNAME, None),
         )
 
+
+class Shuffle(StandardDatasetImplementation):
+    """Computes schema and arrow
+    batches for a dataspec transformed by
+    a user_settings transform
+    """
+
     async def to_arrow(
         self, batch_size: int
     ) -> t.AsyncIterator[pa.RecordBatch]:
-
         arrow_batches = [
             batch async for batch in await self.parent_to_arrow(batch_size=1)
         ]
