@@ -1,9 +1,9 @@
 import typing as t
 
-from sarus_data_spec.manager.ops.asyncio.base import BaseScalarOp
+from sarus_data_spec.manager.ops.asyncio.base import ScalarImplementation
 
 
-class PrivacyParams(BaseScalarOp):
+class PrivacyParams(ScalarImplementation):
     async def value(self) -> t.Any:
         assert self.scalar.is_privacy_params()
         points = self.scalar.protobuf().spec.privacy_params.points
