@@ -1,10 +1,10 @@
 from __future__ import annotations
 
 from typing import (
-    Any,
     AsyncIterator,
     Callable,
     Dict,
+    Iterator,
     List,
     Optional,
     Protocol,
@@ -21,11 +21,21 @@ try:
 except ModuleNotFoundError:
     pass  # Warning is displayed by typing.py
 
+import warnings
+
 from sarus_data_spec.storage.typing import HasStorage
 import sarus_data_spec.protobuf as sp
 import sarus_data_spec.query_manager.typing as sqmt
 import sarus_data_spec.typing as st
 
+try:
+    import sqlalchemy as sa
+
+    sa_engine = sa.engine.Engine
+except ModuleNotFoundError:
+    warnings.warn("Sqlalchemy not installed, cannot send sql queries")
+    sa_engine = t.Any  # type: ignore
+
 
 @runtime_checkable
 class Manager(st.Referrable[sp.Manager], HasStorage, Protocol):
@@ -33,7 +43,7 @@ class Manager(st.Referrable[sp.Manager], HasStorage, Protocol):
 
     def to_arrow(
         self, dataset: st.Dataset, batch_size: int
-    ) -> st.ContextManagerIterator[pa.RecordBatch]:
+    ) -> t.Iterator[pa.RecordBatch]:
         """Synchronous method based on async_to_arrow
         that returns an iterator of arrow batches
         for the input dataset"""
@@ -84,6 +94,20 @@ class Manager(st.Referrable[sp.Manager], HasStorage, Protocol):
         """Prepare all the parents of a Dataspec."""
         ...
 
+    def sql_prepare(self, dataset: st.Dataset) -> None:
+        """Make sure a dataset is sql ready"""
+        ...
+
+    async def async_sql_prepare(self, dataset: st.Dataset) -> None:
+        """Make sure a dataset is sql ready asynchronously."""
+        ...
+
+    async def async_sql_prepare_parents(self, dataset: st.Dataset) -> None:
+        """SQL prepare all the parents of a dataset. It should sql_prepare
+        dataset parents and prepare Scalars parents.
+        """
+        ...
+
     def cache_scalar(self, scalar: st.Scalar) -> None:
         """Synchronous scalar caching"""
         ...
@@ -135,6 +159,12 @@ class Manager(st.Referrable[sp.Manager], HasStorage, Protocol):
     ) -> tf.data.Dataset:
         ...
 
+    def to_sql(self, dataset: st.Dataset) -> None:
+        ...
+
+    async def async_to_sql(self, dataset: st.Dataset) -> None:
+        ...
+
     def status(
         self, dataspec: st.DataSpec, task_name: str
     ) -> t.Optional[st.Status]:
@@ -172,9 +202,38 @@ class Manager(st.Referrable[sp.Manager], HasStorage, Protocol):
     def sql(
         self,
         dataset: st.Dataset,
-        query: str,
+        query: t.Union[str, t.Mapping[t.Union[str, t.Tuple[str, ...]], str]],
         dialect: Optional[st.SQLDialect] = None,
-    ) -> List[Dict[str, Any]]:
+        batch_size: int = 10000,
+    ) -> Iterator[pa.RecordBatch]:
+        ...
+
+    async def async_sql(
+        self,
+        dataset: st.Dataset,
+        query: t.Union[str, t.Mapping[t.Union[str, t.Tuple[str, ...]], str]],
+        dialect: Optional[st.SQLDialect] = None,
+        batch_size: int = 10000,
+    ) -> AsyncIterator[pa.RecordBatch]:
+        ...
+
+    async def execute_sql_query(
+        self,
+        dataset: st.Dataset,
+        caching_properties: t.Mapping[str, str],
+        query: t.Union[str, t.Mapping[t.Union[str, t.Tuple[str, ...]], str]],
+        dialect: t.Optional[st.SQLDialect] = None,
+        batch_size: int = 10000,
+    ) -> t.AsyncIterator[pa.RecordBatch]:
+        ...
+
+    async def async_sql_op(
+        self,
+        dataset: st.Dataset,
+        query: t.Union[str, t.Mapping[t.Union[str, t.Tuple[str, ...]], str]],
+        dialect: t.Optional[st.SQLDialect] = None,
+        batch_size: int = 10000,
+    ) -> t.AsyncIterator[pa.RecordBatch]:
         ...
 
     def is_big_data(self, dataset: st.Dataset) -> bool:
@@ -185,17 +244,33 @@ class Manager(st.Referrable[sp.Manager], HasStorage, Protocol):
         or not"""
         ...
 
+    def is_pushed_to_sql(self, dataspec: st.DataSpec) -> bool:
+        """Returns whether a dataspec should be pushed to sql
+        or not"""
+        ...
+
     def attribute(
         self, name: str, dataspec: st.DataSpec
     ) -> t.Optional[st.Attribute]:
         ...
 
+    def attributes(
+        self, name: str, dataspec: st.DataSpec
+    ) -> t.List[st.Attribute]:
+        ...
+
     def links(self, dataset: st.Dataset) -> st.Links:
         ...
 
     async def async_links(self, dataset: st.Dataset) -> st.Links:
         ...
 
+    def sql_pushing_schema_prefix(self, dataset: st.Dataset) -> str:
+        ...
+
+    def engine(self, uri: str) -> sa_engine:
+        ...
+
 
 @runtime_checkable
 class HasManager(Protocol):
@@ -254,33 +329,7 @@ class DelegatedComputation(Computation, t.Protocol[T]):
 
 @t.runtime_checkable
 class ExternalOpImplementation(t.Protocol):
-    """External PEP op implementation class.
-
-    This class wraps together several elements of an external op
-    implementation:
-        - `data_fn` is the function that computes the output value from the
-          input(s) value(s).
-    """
-
-    data_fn: t.Callable
-    transform_id: t.Optional[str]
-
-    def is_pep_transform(self) -> bool:
-        ...
-
-    def is_dp_transform(self) -> bool:
-        ...
-
-    def dp_equivalent(self) -> t.Optional[DPImplementation]:
-        ...
-
-
-class PEPImplementation(ExternalOpImplementation, t.Protocol):
-    """External PEP op implementation class.
-
-    This class wraps together several elements of an external op
-    implementation `data_fn` is the function that computes the output value
-    from the input(s) value(s).
+    """External Op implementation class.
 
     The `allowed_pep_args` is a list of combinations of arguments' names which
     are managed by the Op. The result of the Op will be PEP only if the set of
@@ -295,16 +344,48 @@ class PEPImplementation(ExternalOpImplementation, t.Protocol):
           dataspecs
         - `a` and `b` are PEP dataspecs, `c` is either not a dataspec or a
           public dataspec
-
-    The `is_token_preserving` attribute is a function that takes as input the
-    non-evaluated arguments and returns a boolean of whether the PEP output
-    token is the same as the PEP input token. An Op that changes the number or
-    order of the rows is not token preserving.
     """
 
+    transform_id: str
     allowed_pep_args: t.List[t.Set[str]]
-    is_token_preserving: t.Callable
 
+    def dataspec(self) -> st.DataSpec:
+        ...
 
-class DPImplementation(ExternalOpImplementation, t.Protocol):
-    allowed_pep_args: t.List[t.Set[str]]
+    def dp_equivalent(self) -> t.Optional[ExternalOpImplementation]:
+        ...
+
+    @staticmethod
+    async def call(*args: t.Any, **kwargs: t.Any) -> t.Any:
+        """Compute the external op output value.
+
+        DP implementation take additional arguments:
+            - `seed` an integer used to parametrize rangom number generators
+            - `budget` the privacy budget that can be spend in the op
+            - `pe` the protected entity information of each row
+        """
+
+    def pep_kind(self, *args: t.Any, **kwargs: t.Any) -> sqmt.PEPKind:
+        """Return the PEP properties of the transform.
+
+        It takes the transform arguments as input because it can depend on some
+        transform parameters. For instance, it is not PEP if we are aggregating
+        the rows (axis=0) and it is PEP if we are aggregating the columns
+        (axis=1).
+        """
+
+    def is_dp(self, *args: t.Any, **kwargs: t.Any) -> bool:
+        """Return True if the DP transform is compatible with the arguments.
+
+        It takes the transform arguments as input because it can depend on some
+        transform parameters. For instance, if we are aggregating the rows
+        (axis=0), then there might be an equivalent DP transform but if we are
+        aggregating the columns there might not (axis=1).
+
+        NB: This function should have the same signature as the call function.
+        """
+
+    async def private_queries(
+        self, *args: t.Any, **kwargs: t.Any
+    ) -> t.List[st.PrivateQuery]:
+        """Return the PrivateQueries summarizing DP characteristics."""
