@@ -140,38 +140,38 @@ def verifies_dp(
     public_context: Collection[str],
     privacy_limit: Optional[st.PrivacyLimit],
 ) -> bool:
+    """Check if a variant constraint satisfies a DP profile.
+
+    For now, return True only for strict equality.
+    """
     if privacy_limit is None:
         raise ValueError(
             "Input privacy limit required when checking against DP."
         )
 
     kind = variant_constraint.constraint_kind()
-    if kind in [st.ConstraintKind.PUBLIC, st.ConstraintKind.SYNTHETIC]:
-        return True
+    if kind != st.ConstraintKind.DP:
+        return False
 
-    elif kind == st.ConstraintKind.DP:
-        constraint_privacy_limit = variant_constraint.privacy_limit()
-        if constraint_privacy_limit is None:
-            raise ValueError(
-                "Found a DP constraint without a privacy limit "
-                "when checking against DP."
-            )
-
-        return cast(
-            bool,
-            privacy_limit.delta_epsilon_dict()
-            == constraint_privacy_limit.delta_epsilon_dict(),
+    constraint_privacy_limit = variant_constraint.privacy_limit()
+    if constraint_privacy_limit is None:
+        raise ValueError(
+            "Found a DP constraint without a privacy limit "
+            "when checking against DP."
         )
 
-    else:
-        return False
+    return cast(
+        bool,
+        privacy_limit.delta_epsilon_dict()
+        == constraint_privacy_limit.delta_epsilon_dict(),
+    )
 
 
 def compile(
     query_manager: QueryManager,
     dataspec: st.DataSpec,
     kind: st.ConstraintKind,
-    public_context: List[str],
+    public_context: Collection[str],
     privacy_limit: Optional[st.PrivacyLimit],
 ) -> Optional[st.DataSpec]:
     """Returns a compliant Node or None."""
@@ -268,7 +268,10 @@ def compile_synthetic(
             sc_syn_args, sc_contexts = zip(*sc_syn_args_context)
         else:
             sc_syn_args, sc_contexts = [], ([],)
-        new_context = list(set(sum(ds_contexts + sc_contexts, [])))
+        new_context = cast(
+            Collection[str],
+            list(set(sum(map(list, ds_contexts + sc_contexts), []))),
+        )
         args, kwargs = nest_args(
             cast(List[st.DataSpec], list(ds_syn_args)),
             cast(List[st.DataSpec], list(sc_syn_args)),
@@ -347,18 +350,18 @@ def compile_dp(
     transform = dataspec.transform()
     if dataspec.prototype() == sp.Dataset:
         dataset = cast(st.Dataset, dataspec)
-        DatasetOpClass = routing.get_dataset_op(transform)
-        is_dp_applicable = DatasetOpClass(dataset).is_dp_applicable(
+        _, DatasetStaticChecker = routing.get_dataset_op(transform)
+        is_dp_applicable = DatasetStaticChecker(dataset).is_dp_applicable(
             public_context
         )
-        dp_transform = DatasetOpClass(dataset).dp_transform()
+        dp_transform = DatasetStaticChecker(dataset).dp_transform()
     else:
         scalar = cast(st.Scalar, dataspec)
-        ScalarOpClass = routing.get_scalar_op(transform)
-        is_dp_applicable = ScalarOpClass(scalar).is_dp_applicable(
+        _, ScalarStaticChecker = routing.get_scalar_op(transform)
+        is_dp_applicable = ScalarStaticChecker(scalar).is_dp_applicable(
             public_context
         )
-        dp_transform = ScalarOpClass(scalar).dp_transform()
+        dp_transform = ScalarStaticChecker(scalar).dp_transform()
 
     if not is_dp_applicable:
         return compile_synthetic(query_manager, dataspec, public_context)
