@@ -2,14 +2,24 @@ from __future__ import annotations
 
 from enum import Enum
 from typing import Collection, List, Optional, Protocol
+import logging
 
 from sarus_data_spec.storage.typing import Storage
 import sarus_data_spec.typing as st
 
+logger = logging.getLogger(__name__)
+
 
 class DataspecPrivacyPolicy(Enum):
     WHITE_LISTED = "Result of whitelisted operations"
     DP = "DP estimate"
+    SYNTHETIC = "Evaluated from synthetic data only"
+
+
+class PEPKind(Enum):
+    NOT_PEP = 0
+    PEP = 1
+    TOKEN_PRESERVING = 2
 
 
 class QueryManager(Protocol):
@@ -20,7 +30,7 @@ class QueryManager(Protocol):
         self,
         dataspec: st.DataSpec,
         kind: st.ConstraintKind,
-        public_context: List[str],
+        public_context: Collection[str],
         privacy_limit: Optional[st.PrivacyLimit],
     ) -> Optional[st.DataSpec]:
         ...
@@ -95,3 +105,22 @@ class QueryManager(Protocol):
         PUBLIC constraint.
         """
         ...
+
+    def private_queries(self, dataspec: st.DataSpec) -> List[st.PrivateQuery]:
+        """Return the list of PrivateQueries used in a Dataspec's transform.
+
+        It represents the privacy loss associated with the current computation.
+
+        It can be used by Sarus when a user (Access object) reads a DP dataspec
+        to update its accountant. Note that Private Query objects are generated
+        with a random uuid so that even if they are submitted multiple times to
+        an account, they are only accounted once (ask @cgastaud for more on
+        accounting)."""
+
+    def is_dp(self, dataspec: st.DataSpec) -> bool:
+        """Return True if the dataspec is the result of a DP transform.
+
+        This is a simple implementation. This function checks if the
+        dataspec's transform has a privacy budget and a random seed as an
+        argument.
+        """
