@@ -11,23 +11,14 @@ from typing import (
     Type,
     cast,
 )
-import json
 import pickle as pkl
+import typing as t
 
 from sarus_data_spec.base import Referring
-from sarus_data_spec.constants import PRIVATE_QUERY
-from sarus_data_spec.protobuf.utilities import dejson
 from sarus_data_spec.transform import Transform
 import sarus_data_spec.protobuf as sp
 import sarus_data_spec.typing as st
 
-try:
-    from sarus_differential_privacy.protobuf.private_query_pb2 import (
-        PrivateQuery,
-    )
-except ModuleNotFoundError:
-    pass  # warning raised in typing.py
-
 
 class Scalar(Referring[sp.Scalar]):
     """A python class to describe scalars"""
@@ -64,6 +55,10 @@ class Scalar(Referring[sp.Scalar]):
         """Is the dataspec a remotely defined dataset."""
         return self.manager().is_remote(self)
 
+    def is_source(self) -> bool:
+        """Is the scalar not composed."""
+        return not self.is_transformed()
+
     def is_model(self) -> bool:
         """Is the scalar a model."""
         return self._protobuf.spec.HasField("model")
@@ -74,7 +69,13 @@ class Scalar(Referring[sp.Scalar]):
 
     def is_random_seed(self) -> bool:
         """Is the scalar a random seed."""
-        return self._protobuf.spec.HasField("random_seed")
+        if self._protobuf.spec.HasField("random_seed"):
+            return True
+        if self.is_transformed():
+            transform = self.transform()
+            if transform.protobuf().spec.HasField("derive_seed"):
+                return True
+        return False
 
     def is_pep(self) -> bool:
         """Is the scalar PEP."""
@@ -97,6 +98,10 @@ class Scalar(Referring[sp.Scalar]):
 
         # collects last status for each task. This if needed will also update
         # them if any synchronization is needed.
+        if type(task_names) not in [list, set, tuple]:
+            raise TypeError(
+                f"Invalid task_names passed to dataset.status {task_names}"
+            )
         statuses = [
             self.manager().status(self, task_name) for task_name in task_names
         ]
@@ -135,41 +140,18 @@ class Scalar(Referring[sp.Scalar]):
 
         return args_parents, kwargs_parents
 
-    def sources(self) -> Set[st.Dataset]:
+    def sources(
+        self, type_name: t.Optional[str] = sp.type_name(sp.Dataset)
+    ) -> Set[st.DataSpec]:
         """Returns the set of non-transformed datasets that are parents
         of the current dataset"""
-
-        class Sources(st.Visitor):
-            visited: Set[st.DataSpec] = set()
-            results: Set[st.Dataset] = set()
-
-            def transformed(
-                self,
-                visited: st.DataSpec,
-                transform: st.Transform,
-                *arguments: st.DataSpec,
-                **named_arguments: st.DataSpec,
-            ) -> None:
-                if visited not in self.visited:
-                    for argument in arguments:
-                        argument.accept(self)
-                    for _, argument in named_arguments.items():
-                        argument.accept(self)
-                    self.visited.add(visited)
-
-            def other(self, visited: st.DataSpec) -> None:
-                if visited.prototype() == sp.Dataset:
-                    dataset = cast(st.Dataset, visited)
-                    self.results.add(dataset)
-
-        visitor = Sources()
-        self.accept(visitor)
-        return visitor.results
+        sources = self.storage().sources(self, type_name=type_name)
+        return sources
 
     def variant(
         self,
         kind: st.ConstraintKind,
-        public_context: List[str] = [],
+        public_context: Collection[str] = (),
         privacy_limit: Optional[st.PrivacyLimit] = None,
     ) -> Optional[st.DataSpec]:
         return (
@@ -181,7 +163,7 @@ class Scalar(Referring[sp.Scalar]):
     def variants(self) -> Collection[st.DataSpec]:
         return self.manager().query_manager().variants(self)
 
-    def private_queries(self) -> List[PrivateQuery]:
+    def private_queries(self) -> List[st.PrivateQuery]:
         """Return the list of PrivateQueries used in a Dataspec's transform.
 
         It represents the privacy loss associated with the current computation.
@@ -191,15 +173,7 @@ class Scalar(Referring[sp.Scalar]):
         with a random uuid so that even if they are submitted multiple times to
         an account, they are only accounted once (ask @cgastaud for more on
         accounting)."""
-        attribute = self.attribute(name=PRIVATE_QUERY)
-        if attribute is None:
-            return []
-
-        private_query_str = attribute[PRIVATE_QUERY]
-        return [
-            cast(PrivateQuery, dejson(q))
-            for q in json.loads(private_query_str)
-        ]
+        return self.manager().query_manager().private_queries(self)
 
     def value(self) -> st.DataSpecValue:
         return self.manager().value(self)
@@ -293,6 +267,9 @@ class Scalar(Referring[sp.Scalar]):
     def attribute(self, name: str) -> Optional[st.Attribute]:
         return self.manager().attribute(name=name, dataspec=self)
 
+    def attributes(self, name: str) -> List[st.Attribute]:
+        return self.manager().attributes(name=name, dataspec=self)
+
 
 # Builders
 def model(
