@@ -17,11 +17,11 @@ import sarus_data_spec.typing as st
 class Schema(Referring[sp.Schema]):
     """A python class to describe schemas"""
 
-    def __init__(self, protobuf: sp.Schema) -> None:
+    def __init__(self, protobuf: sp.Schema, store: bool = True) -> None:
         self._referred = {
             protobuf.dataset
         }  # This has to be defined before it is initialized
-        super().__init__(protobuf)
+        super().__init__(protobuf, store=store)
 
     def prototype(self) -> t.Type[sp.Schema]:
         """Return the type of the underlying protobuf."""
@@ -42,24 +42,17 @@ class Schema(Referring[sp.Schema]):
         """Returns the first type level of the schema"""
         return Type(self.protobuf().type)
 
-    def protected_path(self) -> Path:
-        """Returns the path to the protected entities"""
-        return Path(self.protobuf().protected)
-
     def data_type(self) -> Type:
         """Returns the first type level containing the data,
         hence skips the protected_entity struct if there is one"""
-        type = self.protobuf().type
-        if type.HasField('struct'):
-            keys = [
-                i
-                for i, element in enumerate(type.struct.fields)
-                if element.name == 'data'
-            ]
-            if len(keys) > 0:
-                assert len(keys) == 1
-                return Type(type.struct.fields[keys[0]].type)
-        return Type(type)
+        return self.type().data_type()
+
+    def is_protected(self) -> bool:
+        return self.type().has_protected_format()
+
+    def protected_path(self) -> Path:
+        """Returns the path to the protected entities"""
+        return Path(self.protobuf().protected)
 
     # TODO: Add to_parquet, to_tensorflow, to_sql... here?
     # The Schema has a manager, it would provide the implementation
