@@ -16,7 +16,8 @@ logger = logging.getLogger(__name__)
 
 
 class DataSpecErrorStatus(Exception):
-    def __init__(self, relaunch: bool, error_msg: str) -> None:
+    def __init__(self, relaunch_error_msg: t.Tuple[bool, str]) -> None:
+        relaunch, error_msg = relaunch_error_msg
         self.relaunch = relaunch
         self.error_msg = error_msg
 
@@ -110,9 +111,8 @@ class Status(Referring[sp.Status]):
         proto = self.protobuf()
         # We update its timestamp and properties
         proto.datetime = datetime.now().isoformat()
-        try:
-            stage = proto.task_stages.pop(task)
-        except KeyError:
+        stage = proto.task_stages.pop(task)
+        if stage.WhichOneof('stage') is None:
             return None
         # we can create a new empty status only
         # if the current stage is static ie ready
