@@ -7,6 +7,7 @@ import typing as t
 import numpy as np
 
 from sarus_data_spec.base import Referrable
+from sarus_data_spec.path import straight_path
 import sarus_data_spec.dataset as sd
 import sarus_data_spec.protobuf as sp
 import sarus_data_spec.typing as st
@@ -571,14 +572,22 @@ def cast(type: st.Type) -> Transform:
     )
 
 
-def sample(fraction_size: t.Union[float, int]) -> Transform:
+def sample(fraction_size: t.Union[float, int], seed: st.Scalar) -> Transform:
+    """Transform to sample from a dataspec
+    - the dataset that needs to be protected as the first arg
+    - a kwarg seed"""
     return Transform(
         sp.Transform(
             name='Sample',
             spec=sp.Transform.Spec(
-                sample=sp.Transform.Sample(size=fraction_size)
+                sample=sp.Transform.Sample(
+                    size=fraction_size,
+                    seed=seed.protobuf(),
+                )
                 if isinstance(fraction_size, int)
-                else sp.Transform.Sample(fraction=fraction_size)
+                else sp.Transform.Sample(
+                    fraction=fraction_size, seed=seed.protobuf()
+                )
             ),
             inversible=False,
             schema_preserving=False,
@@ -602,7 +611,7 @@ def user_settings() -> Transform:
     )
 
 
-def automatic_user_settings() -> Transform:
+def automatic_user_settings(max_categories: int = 200) -> Transform:
     """Transform to be called on a protected dataset
     we want to change the schema. It creates a scalar
     whose value explicits the new type of the schema"""
@@ -610,7 +619,9 @@ def automatic_user_settings() -> Transform:
         sp.Transform(
             name='automatic_user_settings',
             spec=sp.Transform.Spec(
-                automatic_user_settings=sp.Transform.AutomaticUserSettings()
+                automatic_user_settings=sp.Transform.AutomaticUserSettings(
+                    max_categories=max_categories
+                )
             ),
             inversible=False,
             schema_preserving=False,
@@ -642,7 +653,7 @@ def protect() -> Transform:
     the public tables"""
     return Transform(
         sp.Transform(
-            name='Dataset with Protected Entity',
+            name='Protect',
             spec=sp.Transform.Spec(protect_dataset=sp.Transform.Protect()),
             inversible=True,
             schema_preserving=False,
@@ -662,7 +673,6 @@ def transcode() -> st.Transform:
 
 
 def inverse_transcode() -> st.Transform:
-
     return Transform(
         sp.Transform(
             name='Inverse Transcoding for synthetic data',
@@ -815,17 +825,76 @@ def group_by_pe() -> st.Transform:
     )
 
 
-def select_sql(query: str) -> st.Transform:
-    """Transform that applies a query to a dataset.
-    Calling .schema() or .to_arrow() on a select_sql transformed dataset
-    the .sql method will be invoked and the query will be executed.
+def differentiated_sample(  # type: ignore[no-untyped-def]
+    fraction_size: t.Union[float, int], seed=st.Scalar
+) -> Transform:
+    return Transform(
+        sp.Transform(
+            name='DifferentiatedSample',
+            spec=sp.Transform.Spec(
+                differentiated_sample=sp.Transform.DifferentiatedSample(
+                    size=fraction_size, seed=seed.protobuf()
+                )
+                if isinstance(fraction_size, int)
+                else sp.Transform.DifferentiatedSample(
+                    fraction=fraction_size, seed=seed.protobuf()
+                )
+            ),
+        )
+    )
+
+
+def select_sql(
+    query: t.Union[str, t.Dict[t.Union[str, t.Tuple[str]], str]],
+    dialect: t.Optional[st.SQLDialect] = None,
+) -> st.Transform:
+    """Transform that applies a query or a batch of aliased queries to
+    a dataset.Calling .schema() or .to_arrow() on a select_sql transformed
+    dataset the .sql method will be invoked and the query will be executed.
     """
+    sql_dialect = (
+        sp.Transform.SelectSql.SQLDialect.POSTGRES
+        if not dialect
+        else sp.Transform.SelectSql.SQLDialect.Value(dialect.name)
+    )
+    if isinstance(query, str):
+        select_sql = sp.Transform.SelectSql(
+            query=query,
+            sql_dialect=sql_dialect,
+        )
+    elif len(query) == 0:
+        raise ValueError(
+            """Transform `SelecltSQL` must be used with
+            at least one query"""
+        )
+    else:
+        queries = {
+            straight_path(
+                list(
+                    name
+                    if isinstance(name, t.Tuple)  # type: ignore
+                    else (name,)
+                )
+            ): qry
+            for (name, qry) in query.items()
+        }
+        select_sql = sp.Transform.SelectSql(
+            aliased_queries=sp.Transform.SelectSql.AliasedQueries(
+                aliased_query=(
+                    sp.Transform.SelectSql.AliasedQuery(
+                        path=_path.protobuf(),
+                        query=qry,
+                    )
+                    for (_path, qry) in queries.items()
+                ),
+            ),
+            sql_dialect=sql_dialect,
+        )
+
     return Transform(
         sp.Transform(
             name='select_sql',
-            spec=sp.Transform.Spec(
-                select_sql=sp.Transform.SelectSql(query=query)
-            ),
+            spec=sp.Transform.Spec(select_sql=select_sql),
             inversible=False,
             schema_preserving=False,
         )
