@@ -3,46 +3,56 @@
 from __future__ import annotations
 
 from abc import abstractmethod
-import typing as t
-import warnings
-
-import pandas as pd
-import pyarrow as pa
-
-try:
-    import tensorflow as tf
-except ModuleNotFoundError:
-    warnings.warn('tensorflow not found, tensorflow datasets not available')
-
-try:
-    import sklearn  # noqa: F401
-except ModuleNotFoundError:
-    warnings.warn('sklearn not found, sklearn models not available')
-
-try:
-    import pandas_profiling  # noqa: F401
-except ModuleNotFoundError:
-    warnings.warn('pandas_profiling not found, ProfileReport not available')
-
-try:
-    from sarus_differential_privacy.protobuf.private_query_pb2 import (
-        PrivateQuery,
-    )
-except ModuleNotFoundError:
-    warnings.warn(
-        'sarus_differential_privacy not found, private_query not available'
-    )
-
 from enum import Enum
 import datetime as dt
+import logging
+import typing as t
 
 import numpy as np
+import pandas as pd
+import pyarrow as pa
 
 from sarus_data_spec.protobuf.typing import Protobuf, ProtobufWithUUID
 import sarus_data_spec.manager.typing as manager_typing
 import sarus_data_spec.protobuf as sp
 import sarus_data_spec.storage.typing as storage_typing
 
+logger = logging.getLogger(__name__)
+
+try:
+    import tensorflow as tf
+except ModuleNotFoundError:
+    logger.warning('tensorflow not found, tensorflow datasets not available')
+
+try:
+    import sklearn  # noqa: F401
+except ModuleNotFoundError:
+    logger.warning('sklearn not found, sklearn models not available')
+
+try:
+    from sarus_differential_privacy.query import (
+        PrivateQuery as RealPrivateQuery,
+    )
+
+    PrivateQuery = RealPrivateQuery
+except ImportError:
+    PrivateQuery = t.Any  # type: ignore
+    logger.warning(
+        "`sarus_differential_privacy` not installed. "
+        "DP primitives not available."
+    )
+
+try:
+    from sarus_query_builder.core.typing import Task as RealTask
+
+    Task = RealTask
+except ImportError:
+    Task = t.Any  # type: ignore
+    logger.warning(
+        "`sarus_query_builder` not installed. DP methods not available."
+    )
+
+
 if t.TYPE_CHECKING:
     from sklearn import svm
 
@@ -197,7 +207,7 @@ class DataSpec(Referring[DS], t.Protocol):
     def variant(
         self,
         kind: ConstraintKind,
-        public_context: t.List[str],
+        public_context: t.Collection[str] = (),
         privacy_limit: t.Optional[PrivacyLimit] = None,
     ) -> t.Optional[DataSpec[DS]]:
         ...
@@ -248,13 +258,20 @@ class DataSpec(Referring[DS], t.Protocol):
         ...
 
     def is_transformed(self) -> bool:
-        """Is the dataset composed."""
+        """Is the dataspec composed."""
+        ...
+
+    def is_source(self) -> bool:
+        """Is the dataspec composed."""
         ...
 
     def is_remote(self) -> bool:
         """Is the dataspec a remotely defined dataset."""
         ...
 
+    def sources(self, type_name: t.Optional[str]) -> t.Set[DataSpec]:
+        ...
+
     def transform(self) -> Transform:
         ...
 
@@ -265,6 +282,11 @@ class DataSpec(Referring[DS], t.Protocol):
         ...
 
     def attribute(self, name: str) -> t.Optional[Attribute]:
+        """Return the attribute with the given name or None if not found."""
+        ...
+
+    def attributes(self, name: str) -> t.List[Attribute]:
+        """Return all the attributes with the given name."""
         ...
 
 
@@ -278,15 +300,9 @@ class Dataset(DataSpec[sp.Dataset], t.Protocol):
     def is_protected(self) -> bool:
         ...
 
-    def is_source(self) -> bool:
-        ...
-
     def is_file(self) -> bool:
         ...
 
-    def sources(self) -> t.Set[Dataset]:
-        ...
-
     def schema(self) -> Schema:
         ...
 
@@ -307,6 +323,9 @@ class Dataset(DataSpec[sp.Dataset], t.Protocol):
     ) -> t.AsyncIterator[pa.RecordBatch]:
         ...
 
+    def to_sql(self) -> None:
+        ...
+
     def spec(self) -> str:
         ...
 
@@ -330,8 +349,11 @@ class Dataset(DataSpec[sp.Dataset], t.Protocol):
         ...
 
     def sql(
-        self, query: str, dialect: t.Optional[SQLDialect] = None
-    ) -> t.List[t.Dict[str, t.Any]]:
+        self,
+        query: t.Union[str, t.Mapping[t.Union[str, t.Tuple[str, ...]], str]],
+        dialect: t.Optional[SQLDialect] = None,
+        batch_size: int = 10000,
+    ) -> t.Iterator[pa.RecordBatch]:
         """Executes the sql method on the dataset"""
         ...
 
@@ -504,7 +526,9 @@ class Status(Referring[sp.Status], t.Protocol):
 
     def owner(
         self,
-    ) -> manager_typing.Manager:  # TODO: Maybe find a better name, but this was shadowing the actual manager of this object.  # noqa: E501
+    ) -> (
+        manager_typing.Manager
+    ):  # TODO: Maybe find a better name, but this was shadowing the actual manager of this object.  # noqa: E501
         ...
 
     def clear_task(self, task: str) -> t.Optional[Status]:
@@ -687,6 +711,10 @@ class Type(HasProtobuf[sp.Type], Frozen, Value, t.Protocol):
         """Returns the name of the underlying protobuf."""
         ...
 
+    def has_protected_format(self) -> bool:
+        """Return True if the Type has the protected formalism."""
+        ...
+
     def latex(self: Type, parenthesized: bool = False) -> str:
         """return a latex representation of the type"""
         ...
@@ -816,6 +844,7 @@ class SQLDialect(Enum):
     ORACLE = 5
     BIG_QUERY = 6
     REDSHIFT = 7
+    HIVE = 8
 
 
 class InferredDistributionName(Enum):
@@ -1319,10 +1348,6 @@ class InferredAlphabet(t.Protocol):
 T = t.TypeVar('T', covariant=True)
 
 
-class ContextManagerIterator(t.Iterator[T], t.ContextManager, t.Protocol):
-    ...
-
-
 class Links(Referring[sp.Links], t.Protocol):
     """A python abstract class to describe all
     the links statistics in a dataset"""
@@ -1360,7 +1385,7 @@ class LinkStatistics(HasProtobuf[sp.Links.LinkStat], t.Protocol):
         ...
 
 
-class PrivacyLimit:
+class PrivacyLimit(t.Protocol):
     """An abstract Privacy Limit class."""
 
     def delta_epsilon_dict(self) -> t.Dict[float, float]:
